#include "epix.h"
#include "sample.h"

using namespace ePiX;

// Represent a sample of specified color, name, mass, and temperature
// e.g. sample(Red(), "He", 0.004, 273.15) or sample(Blue(), "Al", "0.0270")
sample::sample(const Color& color,
	       const std::string& name, double Mx, double Tx)
  : m_color(color),
    m_name(name),
    m_Mx(Mx/6.022e23), // Avogadro's number
    m_Tx(Tx), vmp(0.0), fvmp(0.0) { }

void sample::plot(double Tmin, double Tmax, unsigned int num_pts) const
{
  if (num_pts == 0)
    return;

  // else
  std::vector<P> pts(num_pts+1);
  const double dT((Tmax - Tmin)/num_pts);

  // calculate probability density data
  for (unsigned int i=0; i<=num_pts; ++i)
    {
      const double T(rescale(Tmin + i*dT));
      const double tmp(pow(T, 2));
      const double res(boltzmann() * tmp * exp(-m_Mx * tmp/(2 * k * m_Tx)));

      if (res > fvmp)
	{
	  fvmp = res;
	  vmp = T;
	}

      pts.at(i) = P(T, res);
    }

  plain(m_color); // set line color
  path data(pts, false, false); // neither closed nor filled
  data.draw();

  label();
} // end of sample::plot

  // return approximate top right corner of smallest bounding rectangle
P sample::TR(double Tmin, double Tmax) const
{
  const double dT(0.01*(Tmax - Tmin)); // subdivide into 100 pieces

  // assume we'll be plotted over [0, a] x [0, b]
  double Ymax(fvmp);

  for (unsigned int i=0; i<=100; ++i)
    {
      const double T(rescale(Tmin + i*dT));
      const double tmp(pow(T, 2));
      const double res(boltzmann() * tmp * exp(-m_Mx * tmp/(2 * k * m_Tx)));

      Ymax = max(Ymax, res);
    }

  return P(rescale(Tmax), rescale(Ymax));
}

// functions for internal use
double sample::boltzmann() const
{
  return 4 * M_PI * pow(m_Mx / (2 * M_PI * k * m_Tx), 1.5);
}

void sample::label() const
{
  std::ostringstream o;
  o << m_name
    << " vmp " << vmp << " m/s at "
    << m_Tx << " K (" << fvmp << ")";

  ePiX::label(P(vmp, fvmp), P(2,2), o.str(), tr);
}

// initialize "global" member
const double sample::k(1.381e-23); // Boltzmann's constant

